from django.test import TestCase

from djangocms_frontend.contrib.grid.forms import GridColumnForm, GridContainerForm, GridRowForm


class GridFormTestCase(TestCase):
    def test_grid_container_form(self):
        form = GridContainerForm(
            data={
                "container_type": "container",
                "tag_type": "div",
                "margin_devices": ["xl"],
                "padding_devices": ["xs"],
            }
        )
        self.assertTrue(form.is_valid())
        self.assertIn("container_type", form.instance.config)
        self.assertEqual(form.instance.container_type, "container")

    def test_grid_row_form(self):
        form = GridRowForm(
            data={
                "margin_devices": ["xl"],
                "padding_devices": ["xs"],
                "row_cols_xs": 5,
            }
        )
        self.assertTrue(form.is_valid())
        self.assertIn("row_cols_xs", form.instance.config)
        self.assertIn("row_cols_xxl", form.instance.config)
        self.assertEqual(form.instance.row_cols_xs, 5)

    def test_grid_column_form(self):
        form = GridColumnForm(
            data={
                "margin_devices": ["xl"],
                "padding_devices": ["xs"],
                "text_alignment": "",
                "xs_col": "6",
                "xl_offset": "",
            }
        )

        self.assertTrue(form.is_valid())
        self.assertIn("md_me", form.instance.config)
        self.assertEqual(form.instance.xs_col, 6)



from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase
from django.template import Template, TemplateSyntaxError

from tests.fixtures import TestFixture


class AutoComponentTestCase(TestFixture, CMSTestCase):
    def test_autocomponents_are_auto_detected(self):
        from cms.plugin_pool import plugin_pool

        self.assertIn("AutoHeroPlugin", plugin_pool.plugins)

        plugin = plugin_pool.get_plugin("AutoHeroPlugin")
        model = plugin.model
        form = plugin.form

        self.assertEqual(plugin.name, "My Hero Auto Component")
        self.assertEqual(model.__name__, "AutoHero")
        self.assertIn("title", form.declared_fields)
        self.assertIn("slogan", form.declared_fields)
        self.assertIn("hero_image", form.declared_fields)
        self.assertIn("config", form.declared_fields)  # Inherited from djangocms_frontend.models.FrontendUIItem

        self.assertTrue(plugin.allow_children)

    def test_render_autocomponent(self):
        add_plugin(
            placeholder=self.placeholder,
            plugin_type="AutoHeroPlugin",
            language=self.language,
            config={"title": "My Hero", "slogan": "My Hero Slogan", "hero_image": None},
        )
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)

        # Declarative tags render empty
        self.assertContains(response, 9 * "\n" + '<section class="bg-white dark:bg-gray-900">')
        self.assertContains(response, "My Hero")
        self.assertContains(response, "My Hero Slogan")

    def test_autocomponents_slots_are_created(self):
        from cms.plugin_pool import plugin_pool

        self.assertIn("AutoHeroWithSlotsPlugin", plugin_pool.plugins)

        plugin = plugin_pool.get_plugin("AutoHeroWithSlotsPlugin")
        model = plugin.model
        form = plugin.form
        self.assertEqual(model.__name__, "AutoHeroWithSlots")
        self.assertIn("title", form.declared_fields)
        self.assertIn("slogan", form.declared_fields)
        self.assertIn("hero_image", form.declared_fields)
        self.assertIn("config", form.declared_fields)  # Inherited from djangocms_frontend.models.FrontendUIItem
        self.assertTrue(plugin.allow_children)
        self.assertEqual(plugin.child_classes, ["AutoHeroWithSlotsButtonsPlugin"])

        self.assertIn("AutoHeroWithSlotsButtonsPlugin", plugin_pool.plugins)
        slot_plugin = plugin_pool.get_plugin("AutoHeroWithSlotsButtonsPlugin")

        self.assertEqual(slot_plugin.parent_classes, ["AutoHeroWithSlotsPlugin"])

    def test_split_template_tag(self):
        from djangocms_frontend.templatetags.cms_component import split

        self.assertEqual(split("hero.html"), ["hero.html"])
        self.assertEqual(split("Mixin1|Mixin2"), ["Mixin1", "Mixin2"])
        self.assertEqual(split("hero.html"), ["hero.html"])
        # Common alternative:
        self.assertEqual(split("hero.html, Mixin1, Mixin2", ", "), ["hero.html", "Mixin1", "Mixin2"])
        # Not nice but correct:
        self.assertEqual(split("hero.html, Mixin1, Mixin2", ","), ["hero.html", " Mixin1", " Mixin2"])

        self.assertEqual(
            split("Dark mode <bg-dark>|Light mode <bg-light>"),
            [("bg-dark", "Dark mode"), ("bg-light", "Light mode")],
        )
        self.assertEqual(
            split("Dark mode<bg-dark>|Light mode<bg-light>"),  # No space before < needed
            [("bg-dark", "Dark mode"), ("bg-light", "Light mode")],
        )
        self.assertEqual(
            split("Dark mode <bg-dark|bg-light>"),
            ["Dark mode <bg-dark", "bg-light>"],
        )

    def test_invalid_cms_component_usage_missing_required_argument(self):
        # The {% cms_component %} tag requires a component name.
        invalid_template = "{% load cms_tags %}{% cms_component %}"
        with self.assertRaises(TemplateSyntaxError):
            Template(invalid_template)

    def test_invalid_field_usage_invalid_argument(self):
        # The {% field %} tag requires valid arguments: a field name and a component instance.
        # Here we simulate invalid usage by providing an invalid component.
        invalid_template = "{% load cms_tags %}{% field 'nonexistent_field' component %}"
        with self.assertRaises(TemplateSyntaxError):
            Template(invalid_template)

    def test_multiple_cms_component_tags_error(self):
        # Assuming only one {% cms_component %} tag is allowed per template.
        # This should raise an error if multiple tags are used.
        invalid_template = "{% load cms_tags %}{% cms_component 'Hero' %}{% cms_component 'Footer' %}"
        with self.assertRaises(TemplateSyntaxError):
            Template(invalid_template)

    def test_cms_component_invalid_identifier(self):
        # Test that cms_component tag raises ValueError for invalid identifiers
        from django.template import Context

        from djangocms_frontend.templatetags.cms_component import cms_component

        context = Context({"_cms_components": {"cms_component": []}})

        # Valid identifier should work
        cms_component(context, "valid_name")
        self.assertEqual(len(context["_cms_components"]["cms_component"]), 1)

        # Non-string identifiers should raise ValueError
        with self.assertRaises(ValueError) as cm:
            cms_component(context, 123)
        self.assertIn("component name must be a string.", str(cm.exception))

        # Invalid identifiers should raise ValueError
        with self.assertRaises(ValueError) as cm:
            cms_component(context, "invalid-name")
        self.assertIn("valid Python identifier", str(cm.exception))

        with self.assertRaises(ValueError) as cm:
            cms_component(context, "123invalid")
        self.assertIn("valid Python identifier", str(cm.exception))

        with self.assertRaises(ValueError) as cm:
            cms_component(context, "invalid name")
        self.assertIn("valid Python identifier", str(cm.exception))

        with self.assertRaises(ValueError) as cm:
            cms_component(context, "")
        self.assertIn("valid Python identifier", str(cm.exception))

    def test_component_folder_selection(self):
        from djangocms_frontend.component_pool import find_cms_component_templates

        all_components = find_cms_component_templates("cms_components")
        path_components = find_cms_component_templates("cms_components/ui")
        private_components = find_cms_component_templates("my_components")
        no_components = find_cms_component_templates("no_components")

        assert set(all_components) == {
            ("tests.test_app", "test_app/cms_components/hero.html"),
            ("tests.test_app", "test_app/cms_components/with_slots.html"),
            ("tests.test_app", "test_app/cms_components/ui/button.html"),
        }
        assert path_components == [("tests.test_app", "test_app/cms_components/ui/button.html")]
        assert private_components == [("tests.test_app", "test_app/my_components/test_component.htm")]
        assert no_components == []



from django import forms
from django.test import TestCase

from djangocms_frontend.fields import (
    AttributesField,
    DeviceChoiceField,
    OptionalDeviceChoiceField,
    TagTypeField,
    TagTypeFormField,
)
from djangocms_frontend.settings import DEVICE_CHOICES


class FieldsTestCase(TestCase):
    def test_attributes_field(self):
        field = AttributesField()
        self.assertEqual(field.verbose_name, "Attributes")
        self.assertEqual(field.blank, True)

    def test_tag_type_field(self):
        field = TagTypeField()
        self.assertEqual(field.verbose_name, "Tag type")
        self.assertEqual(field.default, "div")
        self.assertEqual(field.max_length, 255)
        self.assertEqual(
            field.help_text,
            "Select the HTML tag to be used.",
        )

        form_field = TagTypeFormField()
        self.assertEqual(
            form_field.choices,
            [
                ("div", "div"),
                ("section", "section"),
                ("article", "article"),
                ("header", "header"),
                ("footer", "footer"),
                ("aside", "aside"),
            ],
        )

    def test_optional_device_choice_field(self):
        class Form(forms.Form):
            odc = OptionalDeviceChoiceField()
            odc_not_required = OptionalDeviceChoiceField(required=False)

        form = Form(data={"odc": [size for size, _ in DEVICE_CHOICES]})
        self.assertTrue(form.is_valid(), form.errors)
        self.assertEqual(form.cleaned_data["odc"], None)

        form_1 = Form(data={"odc": ["xs", "sm"]})
        self.assertTrue(form_1.is_valid(), form_1.errors)
        self.assertEqual(form_1.cleaned_data["odc"], ["xs", "sm"])

    def test_device_choice_field(self):
        class Form(forms.Form):
            dc = DeviceChoiceField()

        class Form2(forms.Form):
            dc_not_required = DeviceChoiceField(required=False)

        form = Form(data={"dc": ["xs"]})
        self.assertTrue(form.is_valid(), form.errors)
        self.assertEqual(form.cleaned_data["dc"], ["xs"])

        form_1 = Form2(data={})
        self.assertFalse(form_1.is_valid())
        self.assertFormError(form_1, "dc_not_required", ["Please select at least one device size"])



import json
import typing
import uuid

from classytags.arguments import Argument, MultiKeywordArgument
from classytags.core import Options, Tag
from classytags.helpers import AsTag
from cms.models import CMSPlugin
from cms.templatetags.cms_tags import CMSEditableObject, render_plugin
from django import template
from django.conf import settings as django_settings
from django.contrib.contenttypes.models import ContentType
from django.core.serializers.json import DjangoJSONEncoder
from django.db import models
from django.http import HttpRequest
from django.template.defaultfilters import safe
from django.utils.encoding import force_str
from django.utils.functional import Promise
from django.utils.html import conditional_escape, mark_safe
from entangled.forms import EntangledModelFormMixin

from djangocms_frontend import settings
from djangocms_frontend.fields import HTMLsanitized
from djangocms_frontend.helpers import get_related_object as related_object
from djangocms_frontend.models import FrontendUIItem

register = template.Library()


def is_registering_component(context: template.Context) -> bool:
    return (
        "_cms_components" in context
        and "cms_component" in context["_cms_components"]
        and len(context["_cms_components"]["cms_component"]) == 1
    )


def is_inline_editing_active(context: template.Context) -> bool:
    if "request" in context:
        return context["request"].session.get("inline_editing", True)
    return False


def update_component_properties(context: template.Context, key: str, value: typing.Any, append: bool = False) -> None:
    """ "Adds or appends the value to the property "key" of a component during delcaration"""
    args, kwargs = context["_cms_components"]["cms_component"][0]
    if append:
        # Populate slots with plugin_type and verbose_name
        if key in kwargs:
            kwargs[key].append(value)
        else:
            kwargs[key] = [value]
    else:
        kwargs[key] = value
    context["_cms_components"]["cms_component"][0] = (args, kwargs)


@register.simple_tag
def get_attributes(attribute_field, *add_classes):
    """Joins a list of classes with an attributes field and returns all html attributes"""
    additional_classes = set()
    for classes in add_classes:
        if classes:
            additional_classes.update(classes.split() if isinstance(classes, str) else classes)
    attrs = []
    if attribute_field:
        for key, val in attribute_field.items():
            if key.lower() == "class":
                val = " ".join(additional_classes.union(set(val.split())))
            if val:
                attrs.append(f'{key}="{conditional_escape(val)}"')
            else:
                attrs.append(f"{key}")
    if additional_classes and (not attribute_field or "class" not in attribute_field):
        attrs.append(f'class="{conditional_escape(" ".join(additional_classes))}"')
    return mark_safe(" ".join(attrs))


@register.simple_tag(takes_context=True)
def set_html_id(context: template.Context, instance: FrontendUIItem) -> str:
    if instance.html_id is None:
        request = context.get("request")
        if isinstance(request, HttpRequest):
            key = "frontend_plugins_counter"
            counter = getattr(request, key, 0) + 1
            instance.html_id = f"frontend-plugins-{counter}"
            setattr(request, key, counter)
        else:
            instance.html_id = f"uuid4-{uuid.uuid4()}"
    return instance.html_id


@register.filter
def get_related_object(reference):
    return related_object(dict(obj=reference), "obj")


class LazyEncoder(DjangoJSONEncoder):
    def default(self, obj):
        if isinstance(obj, Promise):
            return force_str(obj)
        return super().default(obj)


@register.filter
def json_dumps(data):
    """Converts data to JSON forcing text on lazy gettext translation"""
    return mark_safe(json.dumps(data, cls=LazyEncoder))


@register.filter
def html_safe(data):
    if HTMLsanitized:
        return safe(data)
    return data


@register.filter
def safe_caption(data):
    if data[:3] == "<p>" and data[-4:] == "</p>":
        block = data[3:-4]
        if "<p>" not in block:
            data = block
    if HTMLsanitized:
        return safe(data)
    return data


@register.simple_tag(takes_context=True)
def framework_info(context, item, as_json=True):
    """Retrieves framework_info for plugin either from existing plugin or from
    content type"""
    plugin = context.get("plugin", None)
    if plugin:  # if possible get from plugin
        return (
            mark_safe(json.dumps(plugin.framework_info.get(item, ""), cls=LazyEncoder))
            if as_json
            else plugin.get(item, "")
        )
    framework_info = context.get("framework_info", {})  # already available
    if not framework_info:
        content_type_id = context.get("content_type_id", None)
        if content_type_id:  # Get from content_type
            model_name = ContentType.objects.get(id=content_type_id).model_class().__name__
            framework_info = settings.FRAMEWORK_PLUGIN_INFO.get(model_name, {})
            context["framework_info"] = framework_info  # and store
    return (
        mark_safe(json.dumps(framework_info.get(item, ""), cls=LazyEncoder))
        if as_json
        else framework_info.get(item, "")
    )


@register.inclusion_tag("djangocms_frontend/user_message.html", takes_context=True)
def user_message(context, message):
    """Renders a user message"""
    toolbar = getattr(context.get("request", None), "toolbar", None)
    if settings.SHOW_EMPTY_CHILDREN and toolbar.edit_mode_active:
        return {"message": message}
    return {}


@register.tag
class SlotTag(Tag):
    name = "slot"
    options = Options(
        Argument("slot_name", required=True),
        blocks=[("endslot", "nodelist")],
    )

    def render_tag(self, context, slot_name, nodelist):
        return ""


class DummyPlugin:
    def __init__(self, nodelist, plugin_type, slot_name: str | None = None) -> "DummyPlugin":
        self.nodelist = nodelist
        self.plugin_type = (f"{plugin_type}{slot_name.capitalize()}Plugin") if slot_name else "DummyPlugin"
        if slot_name is None:
            self.parse_slots(nodelist, plugin_type)
        super().__init__()

    def parse_slots(self, nodelist, plugin_type):
        self.slots = [self]
        for node in nodelist:
            if isinstance(node, SlotTag):
                self.slots.append(DummyPlugin(node.nodelist, plugin_type, node.kwargs.get("slot_name")))

    def get_instances(self):
        return self.slots


class Plugin(AsTag):
    name = "plugin"
    options = Options(
        Argument("name", required=True),
        MultiKeywordArgument("kwargs", required=False),
        "as",
        Argument("varname", resolve=False, required=False),
        blocks=[("endplugin", "nodelist")],
    )

    def message(self, message):
        import warnings

        warnings.warn(message, stacklevel=5)
        return f"<!-- {message} -->" if django_settings.DEBUG else ""

    def get_value(self, context, name, kwargs, nodelist):
        from djangocms_frontend.plugin_tag import plugin_tag_pool

        if name not in plugin_tag_pool:
            return self.message(
                f'To use "{name}" with the {{% plugin %}} template tag, add its plugin class to '
                f"the CMS_COMPONENT_PLUGINS setting"
            )
        context.push()
        instance = plugin_tag_pool[name]["defaults"]
        plugin_class = plugin_tag_pool[name]["class"]

        if issubclass(plugin_class.form, EntangledModelFormMixin):
            # Handle entangled forms such as djangocms-frontend's correctly
            for field, value in kwargs.items():
                for container, fields in plugin_class.form._meta.entangled_fields.items():
                    if field in fields:
                        if isinstance(value, models.Model):
                            # Correctly encode references
                            value = {
                                "model": f"{value._meta.app_label}.{value._meta.model_name}",
                                "pk": value.pk,
                            }
                        instance[container][field] = value
                        break
                else:
                    instance[field] = value
        else:
            instance.update(kwargs)
        # Create context
        context["instance"] = plugin_class.model(**instance)
        # Call render method of plugin
        context = plugin_class().render(context, context["instance"], None)
        # Replace inner plugins with the nodelist, i.e. the content within the plugin tag
        context["instance"].child_plugin_instances = DummyPlugin(
            nodelist, context["instance"].plugin_type
        ).get_instances()
        # ... and render
        result = plugin_tag_pool[name]["template"].render(context.flatten())
        context.pop()
        return result


class RenderChildPluginsTag(Tag):
    """
    This template node is used to render child plugins of a plugin
    instance. It allows for selection of certain plugin types.

    e.g.: {% childplugins instance %}

    {% childplugins instance "LinkPlugin" %} will only render child plugins of
    type LinkPlugin.

    {% childplugins instance or %}
        <a href="/about/">About us</a>
    {% endchildplugins %}

    Keyword arguments:
    instance -- instance of the plugin whose children are to be rendered
    plugin_type -- optional argument which if given will result in filtering
        the direct child plugin types that are rendered.
    or -- optional argument which if given will make the template tag a block
        tag whose content is shown if the placeholder is empty
    """

    name = "childplugins"
    options = Options(
        # PlaceholderOptions parses until the "endchildplugins" tag is found if
        # the "or" option is given
        Argument("instance", required=False),
        Argument("plugin_type", required=False),
        Argument("verbose_name", required=False),
        blocks=[("endchildplugins", "nodelist")],
    )

    def render_tag(self, context, instance, plugin_type, verbose_name, nodelist):
        if is_registering_component(context):
            args, kwargs = context["_cms_components"]["cms_component"][0]
            if plugin_type is None:
                # If tag is used, default to allow_children=True
                kwargs.setdefault("allow_children", True)
            if plugin_type and verbose_name:
                update_component_properties(context, "slots", (plugin_type, verbose_name), append=True)
            context["_cms_components"]["cms_component"][0] = (args, kwargs)

        if not instance:
            instance = context.get("instance", None)

        context.push()
        context["parent"] = instance
        content = []
        if plugin_type and not plugin_type.endswith("Plugin"):
            plugin_type = f"{instance.__class__.__name__}{plugin_type.capitalize()}Plugin"
        for child in getattr(instance, "child_plugin_instances", []):
            if plugin_type is None or child.plugin_type == plugin_type:
                if isinstance(child, DummyPlugin):
                    content.append(child.nodelist.render(context))
                else:
                    content.append(render_plugin(context, child))
        content = "".join(content) or getattr(instance, "simple_content", "")
        if not content.strip() and nodelist:
            # "or" parameter given
            return nodelist.render(context)

        context.pop()
        return content


class InlineField(CMSEditableObject):
    name = "inline_field"
    options = Options(
        Argument("instance"),
        Argument("attribute", default=None, required=False),
        Argument("language", default=None, required=False),
        Argument("filters", default=None, required=False),
        Argument("view_url", default=None, required=False),
        Argument("view_method", default=None, required=False),
        "as",
        Argument("varname", required=False, resolve=False),
    )

    def render_tag(self, context, instance, attribute, **kwargs):
        if isinstance(instance, str) and attribute is None:
            # Shortcut {% inline_field "string" %}
            attribute = instance  # First parameter is the attribute
            instance = context.get("instance", None)  # Use instance from context

        if is_registering_component(context) and attribute:
            # Autodetect inline field and add it to the component
            update_component_properties(context, "frontend_editable_fields", attribute, append=True)
        elif is_inline_editing_active(context) and isinstance(instance, CMSPlugin) and instance.pk:
            # Only allow inline field to be rendered if inline editing is active and the instance is a CMSPlugin
            # DummyPlugins of the ``plugin`` tag are cannot be edited (they have no pk in their model class)
            kwargs["edit_fields"] = attribute
            return super().render_tag(context, instance=instance, attribute=attribute, **kwargs)
        else:
            return getattr(instance, attribute, "")

    def _get_editable_context(
        self, context, instance, language, edit_fields, view_method, view_url, querystring, editmode=True
    ):
        # Fix a not-so-clean solution in django CMS' core: While the template engine checks if an attribute is
        # callable, python expects get_plugin_name to be a method. This is a workaround to make it a method.
        context = super()._get_editable_context(
            context, instance, language, edit_fields, view_method, view_url, querystring, editmode
        )
        if hasattr(instance, "get_plugin_name") and isinstance(instance.get_plugin_name, str):
            value = str(instance.get_plugin_name)
            instance.get_plugin_name = lambda: value
        return context


register.tag(Plugin.name, Plugin)
register.tag(RenderChildPluginsTag.name, RenderChildPluginsTag)
register.tag(InlineField.name, InlineField)



from django.test import TestCase

from djangocms_frontend.component_pool import components


class ComponentPluginTestCase(TestCase):
    def test_component_description(self):
        MyHero, MyHeroPlugin, _ = components["MyHero"]

        instance = MyHero.objects.create()
        instance.initialize_from_form(MyHeroPlugin.form)

        self.assertEqual(instance.get_short_description(), "my title")



from importlib import import_module

from djangocms_frontend import settings

from .attributes import AttributesFormMixin, AttributesMixin
from .title import TitleFormMixin, TitleMixin

__common = {
    "background": ("BackgroundFormMixin", "BackgroundMixin"),
    "responsive": ("ResponsiveFormMixin", "ResponsiveMixin"),
    "sizing": ("SizingFormMixin", "SizingMixin"),
    "spacing": (
        "SpacingFormMixin",
        "SpacingMixin",
        "MarginFormMixin",
        "MarginMixin",
        "PaddingFormMixin",
        "PaddingMixin",
    ),
}

for module, classes in __common.items():
    try:
        module = import_module(f"{__name__}.{settings.framework}.{module}", module)
        for cls in classes:
            globals()[cls] = getattr(module, cls)
    except ModuleNotFoundError:  # pragma: no cover
        for cls in classes:
            globals()[cls] = type(cls, (object,), {})

__all__ = [
    "TitleMixin",
    "TitleFormMixin",
    "AttributesMixin",
    "AttributesFormMixin",
    "BackgroundFormMixin",
    "BackgroundMixin",
    "ResponsiveFormMixin",
    "ResponsiveMixin",
    "SizingFormMixin",
    "SizingMixin",
    "SpacingFormMixin",
    "SpacingMixin",
    "MarginFormMixin",
    "MarginMixin",
    "PaddingFormMixin",
    "PaddingMixin",
]



from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase
from django.test import override_settings

from djangocms_frontend.contrib.carousel.cms_plugins import (
    CarouselPlugin,
    CarouselSlidePlugin,
)
from djangocms_frontend.contrib.carousel.forms import CarouselForm, CarouselSlideForm

from ..fixtures import TestFixture
from ..helpers import get_filer_image


class CarouselPluginTestCase(TestFixture, CMSTestCase):
    def setUp(self):
        super().setUp()
        self.image = get_filer_image()

    def tearDown(self):
        super().tearDown()
        self.image.delete()

    def test_carousel_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=CarouselPlugin.__name__,
            language=self.language,
        )
        plugin.initialize_from_form(CarouselForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(
            ('<div class="slide carousel"' in response.content.decode("utf-8"))
            or ('<div class="carousel slide"' in response.content.decode("utf-8")),
            f'<div class="slide carousel" not found in {response.content.decode("utf-8")}',
        )
        self.assertContains(response, 'data-bs-interval="5000"')
        self.assertContains(response, 'data-bs-keyboard="true"')
        self.assertContains(response, 'data-bs-pause="hover"')
        self.assertContains(response, 'data-bs-ride="carousel"')
        self.assertContains(response, 'data-bs-wrap="true"')

    def test_carousel_slide_plugin(self):
        row = add_plugin(
            placeholder=self.placeholder,
            plugin_type=CarouselPlugin.__name__,
            language=self.language,
        )
        row.initialize_from_form(CarouselForm).save()
        plugin = add_plugin(
            target=row,
            placeholder=self.placeholder,
            plugin_type=CarouselSlidePlugin.__name__,
            language=self.language,
            config=dict(carousel_image=dict(pk=self.image.id, model="filer.Image")),
        )
        plugin.initialize_from_form(CarouselSlideForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(
            ('<div class="slide carousel"' in response.content.decode("utf-8"))
            or ('<div class="carousel slide"' in response.content.decode("utf-8")),
            f'<div class="slide carousel" not found in {response.content.decode("utf-8")}',
        )

        # testing aspect ratio variant
        # also testing multiply entries
        row = add_plugin(
            placeholder=self.placeholder,
            plugin_type=CarouselPlugin.__name__,
            language=self.language,
            config=dict(
                carousel_aspect_ratio="16x9",
            ),
        )
        row.initialize_from_form(CarouselForm).save()
        add_plugin(
            target=row,
            placeholder=self.placeholder,
            plugin_type=CarouselSlidePlugin.__name__,
            language=self.language,
            config=dict(carousel_image=dict(pk=self.image.id, model="filer.Image")),
        ).initialize_from_form(CarouselSlideForm).save()
        add_plugin(
            target=row,
            placeholder=self.placeholder,
            plugin_type=CarouselSlidePlugin.__name__,
            language=self.language,
            config=dict(carousel_image=dict(pk=self.image.id, model="filer.Image")),
        ).initialize_from_form(CarouselSlideForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(
            ('<div class="slide carousel"' in response.content.decode("utf-8"))
            or ('<div class="carousel slide"' in response.content.decode("utf-8")),
            f'<div class="slide carousel" not found in {response.content.decode("utf-8")}',
        )

        # Now testing if links are working
        row = add_plugin(
            placeholder=self.placeholder,
            plugin_type=CarouselPlugin.__name__,
            language=self.language,
        )
        row.initialize_from_form(CarouselForm).save()
        plugin = add_plugin(
            target=row,
            placeholder=self.placeholder,
            plugin_type=CarouselSlidePlugin.__name__,
            language=self.language,
            config=dict(
                carousel_image=dict(pk=self.image.id, model="filer.Image"),
                link=dict(external_link="https://www.divio.com"),
            ),
        )
        plugin.initialize_from_form(CarouselSlideForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'href="https://www.divio.com"')

    @override_settings(
        DJANGOCMS_FRONTEND_CAROUSEL_TEMPLATES=(
            ("default", "Default"),
            ("custom", "Custom"),
        )
    )
    def test_carousel_slide_inherits_parent_template(self):
        """Test that CarouselSlidePlugin correctly retrieves parent's template."""
        # Create a carousel with a custom template
        carousel = add_plugin(
            placeholder=self.placeholder,
            plugin_type=CarouselPlugin.__name__,
            language=self.language,
            config=dict(template="custom"),
        )
        carousel.initialize_from_form(CarouselForm).save()

        # Create a slide as child of the carousel
        slide = add_plugin(
            target=carousel,
            placeholder=self.placeholder,
            plugin_type=CarouselSlidePlugin.__name__,
            language=self.language,
            config=dict(carousel_image=dict(pk=self.image.id, model="filer.Image")),
        )
        slide.initialize_from_form(CarouselSlideForm).save()

        # Get the plugin instance
        plugin_instance = CarouselSlidePlugin()

        # Test that get_render_template uses the parent's template
        template_path = plugin_instance.get_render_template(context={}, instance=slide, placeholder=self.placeholder)

        # The template path should use 'custom' from the parent, not 'default'
        self.assertIn("custom", template_path)
        self.assertEqual(template_path, "djangocms_frontend/bootstrap5/carousel/custom/slide.html")



from django import template
from django.templatetags.static import static
from django.utils.safestring import mark_safe

from djangocms_frontend.contrib.icon.conf import ICON_LIBRARIES

register = template.Library()


@register.inclusion_tag("djangocms_frontend/icon/add_css.html", takes_context=True)
def add_css_for_icon(context, icon):
    if icon and icon.get("library", "") in ICON_LIBRARIES:
        css_link = ICON_LIBRARIES[icon.get("library")][1]
        if css_link:
            if "/" not in css_link:  # static link?
                css_link = static(f"djangocms_frontend/icon/vendor/assets/stylesheets/{css_link}")
            context["icon_css"] = css_link
    return context


@register.simple_tag(takes_context=True)
def icon(context, icon):
    if icon:
        icon_class = icon.get("iconClass", "")
        icon_text = icon.get("iconText", "")
        return mark_safe(f'<i class="{icon_class}">{icon_text}</i>')
    return ""



from unittest.mock import Mock

from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase
from django.http import HttpRequest
from django.test import RequestFactory

from djangocms_frontend import settings
from djangocms_frontend.contrib.link.cms_plugins import TextLinkPlugin
from djangocms_frontend.contrib.link.forms import LinkForm

from ..fixtures import TestFixture


class LinkPluginTestCase(TestFixture, CMSTestCase):
    def test_plugin(self):
        add_plugin(
            placeholder=self.placeholder,
            plugin_type=TextLinkPlugin.__name__,
            language=self.language,
            config=dict(
                link=dict(external_link="https://www.divio.com"),
            ),
        ).initialize_from_form(LinkForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'href="https://www.divio.com"')

        # add more options
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=TextLinkPlugin.__name__,
            language=self.language,
            config=dict(
                dict(link=dict(external_link="https://www.divio.com")),
                link_context="primary",
                link_size="btn-sm",
                link_block=True,
                name="django CMS rocks!",
            ),
        )
        plugin.initialize_from_form(LinkForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "link-primary")
        self.assertContains(response, "btn-sm")
        self.assertContains(response, "d-block")

        # alternate version for link_type
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=TextLinkPlugin.__name__,
            language=self.language,
            config=dict(
                link=dict(internal_link=f"cms.page:{self.page.id}"),
                link_context="primary",
                link_type="btn",
                name="django CMS rocks!",
            ),
        )
        plugin.initialize_from_form(LinkForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "btn-primary")
        self.assertContains(response, 'href="/content/"')

        # alternate version broken link
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=TextLinkPlugin.__name__,
            language=self.language,
            config=dict(
                internal_link=dict(model="cms.page", pk=-3141),
                link_context="primary",
                link_type="btn",
                link_outline=True,
                name="django CMS rocks!",
            ),
        )
        plugin.initialize_from_form(LinkForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "\ndjango CMS rocks")

        # alternate version using link_outline
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=TextLinkPlugin.__name__,
            language=self.language,
            config=dict(
                link=dict(external_link="https://www.divio.com"),
                link_context="primary",
                link_type="btn",
                link_outline=True,
            ),
        )
        plugin.initialize_from_form(LinkForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(
            ('class="btn btn-outline-primary"' in response.content.decode("utf-8"))
            or ('class="btn-outline-primary btn"' in response.content.decode("utf-8")),
            f'Cound not find class="btn btn-outline-primary" in {response.content.decode("utf-8")}',
        )

    def test_link_form(self):
        request = HttpRequest()
        request.POST = {
            "template": settings.LINK_TEMPLATE_CHOICES[0][0],
            "link_type": "link",
        }
        form = LinkForm(request.POST)
        self.assertFalse(form.is_valid())
        request.POST.update(
            {
                "name": "One of the world's most advanced open source CMS",
                "anchor": "allowed-here",
                "margin_devices": ["xs"],
                "padding_devices": ["xs"],
                **(
                    dict(url_grouper=str(self.create_url(manual_url="https://www.django-cms.org/").id))
                    if hasattr(self, "create_url")
                    else dict(
                        link_0="external_link",
                        link_1="https://www.django-cms.org/",
                    )
                ),
            }
        )
        form = LinkForm(request.POST)
        self.assertTrue(form.is_valid(), f"{form.__class__.__name__}:form errors: {form.errors}")
        if hasattr(self, "create_url"):
            self.delete_urls()

    def test_link_form_add_view_name_not_required_by_default(self):
        """Test that name is not required in add view when not child of TextPlugin"""
        plugin = TextLinkPlugin()
        request = RequestFactory().get("/")
        request.user = self.superuser

        # No parent set, so name_required should be False
        plugin._cms_initial_attributes = {}
        form_class = plugin.get_form(request, obj=None, change=False)

        # Instantiate form to check field requirement
        form = form_class()
        # name_required should be False when no parent
        self.assertFalse(getattr(form, "name_required", False))
        self.assertFalse(form.fields["name"].required)

    def test_link_form_add_view_name_required_for_text_plugin_child(self):
        """Test that name is required in add view when child of TextPlugin"""
        plugin = TextLinkPlugin()
        request = RequestFactory().get("/")
        request.user = self.superuser

        # Mock a TextPlugin parent
        parent = Mock()
        parent.plugin_type = "TextPlugin"
        plugin._cms_initial_attributes = {"parent": parent}

        # Get form for add view with TextPlugin parent
        form_class = plugin.get_form(request, obj=None, change=False)

        # Instantiate form to check field requirement
        form = form_class()
        # name_required should be True when parent is TextPlugin
        self.assertTrue(getattr(form, "name_required", False))
        # Verify the field requirement is set correctly
        self.assertTrue(form.fields["name"].required)

    def test_link_form_change_view_name_required_for_text_plugin_child(self):
        """Test that form validation fails when name is required for TextPlugin child"""
        request = HttpRequest()
        request.POST = {
            "template": settings.LINK_TEMPLATE_CHOICES[0][0],
            "link_type": "link",
            "margin_devices": ["xs"],
            "padding_devices": ["xs"],
            **(
                dict(url_grouper=str(self.create_url(manual_url="https://www.django-cms.org/").id))
                if hasattr(self, "create_url")
                else dict(
                    link_0="external_link",
                    link_1="https://www.django-cms.org/",
                )
            ),
            # name field is empty
        }
        form = LinkForm(request.POST)
        form.name_required = True  # Simulate child of TextPlugin
        # Update field requirement as get_form() would do
        form.fields["name"].required = form.name_required
        # Form should be invalid without name when child of TextPlugin
        self.assertFalse(form.is_valid())
        self.assertIn("name", form.errors)
        if hasattr(self, "create_url"):
            self.delete_urls()

    def test_link_form_change_view_name_not_required_without_text_plugin_parent(self):
        """Test that form validation passes when name is not required"""
        request = HttpRequest()
        request.POST = {
            "template": settings.LINK_TEMPLATE_CHOICES[0][0],
            "link_type": "link",
            "margin_devices": ["xs"],
            "padding_devices": ["xs"],
            **(
                dict(url_grouper=str(self.create_url(manual_url="https://www.django-cms.org/").id))
                if hasattr(self, "create_url")
                else dict(
                    link_0="external_link",
                    link_1="https://www.django-cms.org/",
                )
            ),
            # name field is empty
        }
        form = LinkForm(request.POST)
        form.name_required = False  # Not a child of TextPlugin
        # Form should be valid without name when not child of TextPlugin
        self.assertTrue(form.is_valid(), f"Form errors: {form.errors}")
        if hasattr(self, "create_url"):
            self.delete_urls()



# Import the components from the current directory's models module
from cms.plugin_base import CMSPluginBase
from django.utils.encoding import force_str

from djangocms_frontend.helpers import get_related
from djangocms_frontend.models import AbstractFrontendUIItem

try:
    from cms.admin.placeholderadmin import PlaceholderAdmin

    if hasattr(PlaceholderAdmin, "edit_field"):
        # FrontendEditable functionality already implemented in core?
        class FrontendEditableAdminMixin:
            pass
    else:
        # If not use our own version of the plugin-enabled mixin
        raise ImportError
except ImportError:
    # django CMS 3 did not implement this: use our own version of the plugin-enabled mixin
    from .helpers import FrontendEditableAdminMixin

    class PlaceholderAdmin:
        pass


class CMSUIPluginBase(FrontendEditableAdminMixin, CMSPluginBase):
    render_template = "djangocms_frontend/html_container.html"
    change_form_template = "djangocms_frontend/admin/base.html"

    def __str__(self):
        return force_str(super().__str__())

    def render(self, context, instance, placeholder):
        if isinstance(instance, AbstractFrontendUIItem):
            for key, value in instance.config.items():
                if isinstance(value, dict) and set(value.keys()) == {"pk", "model"}:
                    if key not in instance.__dir__():  # hasattr would return the value in the config dict
                        setattr(instance.__class__, key, get_related(key))
            if "instance" not in instance.config and isinstance(instance.config, dict):
                context.update(instance.config)
        return super().render(context, instance, placeholder)

    if not hasattr(PlaceholderAdmin, "edit_field"):
        # If the PlaceholderAdmin does not have the edit_field method, we need to provide
        # the urls and the object getter here
        def get_plugin_urls(self):
            from django.urls import re_path

            info = f"{self.model._meta.app_label}_{self.model._meta.model_name}"

            def pat(regex, fn):
                return re_path(regex, fn, name=f"{info}_{fn.__name__}")

            return [
                pat(r"edit-field/([0-9]+)/([a-z\-]+)/$", self.edit_field),
            ] + super().get_plugin_urls()

        def _get_object_for_single_field(self, object_id, language):
            # Method to get the object for the single field edit view
            from .models import FrontendUIItem

            return FrontendUIItem.objects.get(pk=object_id)


class CMSUIComponent(CMSUIPluginBase):
    pass



from __future__ import annotations

import re

from django import forms, template

register = template.Library()

_TUPLE_RE = re.compile(r"^(.*?)\s*<([^>]+)>\s?$")


@register.simple_tag(takes_context=True)
def cms_component(context: template.Context, *args, **kwargs: dict) -> str:
    """
    Declare a CMS frontend component in a Django template.

    This template tag triggers the generation of a CMS frontend component
    class based on the implate it is part of. The component class is generated
    with the provided arguments as ``Meta`` properties.

    Args:
        context (template.Context): The Django template context object where
            components are stored.
        name: Internal name of the component. Must be unique throught a project.
        **kwargs: Keyword arguments to be added to component's ``Meta`` class list.

    Returns:
        str: An empty string, as this function is intended for side effects only.
    """
    if "_cms_components" in context:
        if len(args) != 1:  # pragma: no cover
            raise ValueError("The cms_component tag requires exactly one positional argument: the component name.")
        if not isinstance(args[0], str):
            raise ValueError("The component name must be a string.")
        if not args[0].isidentifier():
            raise ValueError("The component name must be a valid Python identifier.")
        context["_cms_components"]["cms_component"].append((args, kwargs))
    return ""


@register.simple_tag(takes_context=True)
def field(context: template.Context, field_name: str, field_type: forms.Field, **kwargs) -> str:
    """
    Declares a field entry for the component. It only has an effect if used in
    conjunction with ``{% cms_component %}``.

    Args:
        context (template.Context): The template context object that may contain
            the "_cms_components" dictionary.
        field_name (str): The name of the field to be added to the component.
        field_type (forms.Field): A Django form field class to be used for the field.
        **kwargs: Arbitrary keyword arguments representing field attributes or data.

    Returns:
        str: An empty string, as this function is intended for side effects only.
    """
    if "_cms_components" in context:
        context["_cms_components"]["fields"].append(([field_name, field_type], kwargs))
    return ""


def _to_tuple_if_needed(value: str) -> str | tuple[str, str]:
    """
    Helper function to convert a string into a tuple if it contains a delimiter.

    Args:
        value (str): The string to be converted.

    Returns:
        str | tuple[str, str]: A tuple containing the two parts of the string if it contains
                               a delimiter, otherwise returns the original string.
    """
    match = _TUPLE_RE.fullmatch(value)
    if match:
        return (match.group(2).strip(), match.group(1).strip())
    return value


@register.filter
def split(value: str, delimiter: str = "|") -> list[str | tuple[str, str]]:
    """
    Helper that splits a given string into a list of substrings based on a specified delimiter.
    If the substring is of the format "Verbose name <value>" it is turned into a 2-tuple
    as used by a form field's choices argument.

    Args:
        value (str): The string to be split.
        delimiter (str, optional): The delimiter to use for splitting the string. Defaults to "|".

    Returns:
        list[str | tuple[str, str]: A list of substrings or 2-tuples obtained by splitting the
        input string using the delimiter.
    """
    split_list = value.split(delimiter)
    return [_to_tuple_if_needed(item) for item in split_list]



from django.utils.translation import gettext as _

from djangocms_frontend.contrib.link.helpers import GetLinkMixin

# 'link' type is added manually as it is only required for this plugin
from djangocms_frontend.models import FrontendUIItem
from djangocms_frontend.settings import COLOR_STYLE_CHOICES

COLOR_STYLE_CHOICES = (("link", _("Link")),) + COLOR_STYLE_CHOICES


class Link(GetLinkMixin, FrontendUIItem):
    """
    Components > "Button" Plugin
    https://getbootstrap.com/docs/5.0/components/buttons/
    """

    class Meta:
        proxy = True
        verbose_name = _("Link")

    def get_short_description(self):
        if self.config.get("name", "") and self.get_link():
            return f"{self.name} ({self.get_link()})"
        return self.config.get("name", "") or self.get_link() or _("<link is missing>")



from django.conf import settings
from django.utils.translation import gettext_lazy as _

DEVICE_CHOICES = (
    ("xs", _("Extra small")),  # default <576px
    ("sm", _("Small")),  # default ≥576px
    ("md", _("Medium")),  # default ≥768px
    ("lg", _("Large")),  # default ≥992px
    ("xl", _("Extra large")),  # default ≥1200px
    ("xxl", _("Extra-extra large")),  # default ≥1200px
)
DEVICE_SIZES = tuple(size for size, name in DEVICE_CHOICES)

COLOR_STYLE_CHOICES = getattr(
    settings,
    "DJANGOCMS_FRONTEND_COLOR_STYLE_CHOICES",
    (
        ("default", _("Default")),
        ("alternative", _("Alternative")),
        ("green", _("Green")),
        ("red", _("Red")),
        ("yellow", _("Yellow")),
        ("purple", _("Purple")),
        ("light", _("Light")),
        ("dark", _("Dark")),
    ),
)

SPACER_PROPERTY_CHOICES = (
    ("m", "margin"),
    ("p", "padding"),
)

SPACER_SIDE_CHOICES = (
    ("", "*"),
    ("t", "*-top"),
    ("r", "*-right"),
    ("b", "*-bottom"),
    ("l", "*-left"),
    ("x", "*-left & *-right"),
    ("y", "*-top & *-bottom"),
)

SPACER_X_SIDES_CHOICES = (
    ("x", _("Both")),
    ("s", _("Left")),
    ("e", _("Right")),
)

SPACER_Y_SIDES_CHOICES = (
    ("y", _("Both")),
    ("t", _("Top")),
    ("b", _("Bottom")),
)

SPACER_SIZE_CHOICES = getattr(
    settings,
    "DJANGOCMS_FRONTEND_SPACER_SIZES",
    (
        ("0", "* 0"),
        ("1", "* .25"),
        ("2", "* .5"),
        ("3", "* 1"),
        ("4", "* 1.5"),
        ("5", "* 3"),
    ),
)

SIZE_X_CHOICES = getattr(
    settings,
    "DJANGOCMS_FRONTEND_SIZE_X_CHOICES",
    (
        ("25", "25%"),
        ("50", "50%"),
        ("75", "75%"),
        ("100", "100%"),
        ("auto", _("Auto")),
        ("vw-100", _("Screen")),
    ),
)

SIZE_Y_CHOICES = getattr(
    settings,
    "DJANGOCMS_FRONTEND_SIZE_Y_CHOICES",
    (
        ("25", "25%"),
        ("50", "50%"),
        ("75", "75%"),
        ("100", "100%"),
        ("auto", _("Auto")),
        ("min-vh-100", _("Screen (minimum)")),
    ),
)

NAVBAR_DESIGNS = ()



from __future__ import annotations

import copy
import importlib
import warnings

from cms.plugin_pool import plugin_pool
from cms.templatetags.cms_tags import render_plugin
from django.conf import settings
from django.contrib.admin.sites import site as admin_site
from django.template import engines
from django.template.library import SimpleNode
from django.template.loader import get_template

django_engine = engines["django"]

plugin_tag_pool = {}


IGNORED_FIELDS = (
    "id",
    "cmsplugin_ptr",
    "language",
    "plugin_type",
    "position",
    "creation_date",
    "ui_item",
)


def _get_plugindefaults(instance):
    defaults = {
        field.name: getattr(instance, field.name)
        for field in instance._meta.fields
        if field.name not in IGNORED_FIELDS and bool(getattr(instance, field.name))
    }
    defaults["plugin_type"] = instance.__class__.__name__
    return defaults


class _DummyUser:
    is_superuser = True
    is_staff = True


class _DummyRequest:
    user = _DummyUser()


def render_dummy_plugin(context, dummy_plugin):
    return dummy_plugin.nodelist.render(context)


def patch_template(template):
    """Patches the template to use the dummy plugin renderer instead of the real one."""
    copied_template = copy.deepcopy(template)
    patch = False
    for node in copied_template.template.nodelist.get_nodes_by_type(SimpleNode):
        if node.func == render_plugin:
            patch = True
            node.func = render_dummy_plugin
    return copied_template if patch else template


def get_plugin_class(settings_string: str | type) -> type:
    """Get the plugin class from the settings string or import it if it's a dotted path."""
    if isinstance(settings_string, str):
        if "." in settings_string:
            # import the class if a dotted path is given - raise can exception if not found
            module_name, class_name = settings_string.rsplit(".", 1)
            return getattr(importlib.import_module(module_name), class_name)
        # Get the plugin class from the plugin pool by its name
        return plugin_pool.get_plugin(settings_string)
    return settings_string


def setup():
    allowed_plugin_types = tuple(get_plugin_class(cls) for cls in getattr(settings, "CMS_COMPONENT_PLUGINS", []))

    for plugin in plugin_pool.plugins.values():  # We'll check all plugins irrespectivly of placeholder config
        if not issubclass(plugin, allowed_plugin_types):
            continue
        tag_name = plugin.__name__.lower()
        if tag_name.endswith("plugin"):
            tag_name = tag_name[:-6]
        try:
            instance = plugin.model()  # Create instance with defaults
            plugin_admin = plugin(admin_site=admin_site)
            if hasattr(instance, "initialize_from_form"):
                instance.initialize_from_form(plugin.form)
            if tag_name not in plugin_tag_pool:
                template = get_template(plugin_admin._get_render_template({"request": None}, instance, None))
                plugin_tag_pool[tag_name] = {
                    "defaults": {
                        **_get_plugindefaults(instance),
                        **dict(plugin_type=plugin.__name__),
                    },
                    "template": patch_template(template),
                    "class": plugin,
                }
            else:  # pragma: no cover
                warnings.warn(
                    f'Duplicate candidates for {{% plugin "{tag_name}" %}} found. '
                    f"Only registered {plugin_tag_pool[tag_name]['class'].__name__}.",
                    stacklevel=1,
                )
        except Exception as exc:  # pragma: no cover
            warnings.warn(f"{plugin.__name__}: \n{str(exc)}", stacklevel=1)



from aldryn_client import forms


class Form(forms.BaseForm):
    grid_size = forms.NumberField("Maximum columns to support, default is 12.", required=False)
    enable_icons = forms.CheckboxField(
        "Enable icon support",
        required=False,
        initial=True,
    )

    def to_settings(self, data, settings):
        if data["grid_size"]:
            settings["DJANGOCMS_FRONTEND_GRID_SIZE"] = int(data["grid_size"])
        if data["enable_icons"]:
            settings["DJANGOCMS_FRONTEND_USE_ICONS"] = int(data["enable_icons"])

        return settings



from django.db import migrations


def convert_item(config, direction):
    """Convert FrontendUIItem config to new djangocms_link format."""
    if direction == "forward":
        if config.get("external_link"):
            if (config.get("anchor") or "").strip():
                anchor = "#" + config.get("anchor").strip()
                del config["anchor"]
            else:
                anchor = ""
            config["link"] = {"external_link": config.get("external_link") + anchor}
            del config["external_link"]
            return True
        if config.get("internal_link"):
            model = config.get("internal_link").get("model")
            pk = config.get("internal_link").get("pk")
            config["link"] = {"internal_link": f"{model}:{pk}"}
            if config.get("anchor"):
                config["link"]["anchor"] = "#" + config.get("anchor")
                del config["anchor"]
            del config["internal_link"]
            return True
        if config.get("file_link"):
            config["link"] = {"file_link": config.get("file_link").get("pk")}
            del config["file_link"]
            return True
        if config.get("phone"):
            config["link"] = {"external_link": f"tel:{config.get('phone')}"}
            del config["phone"]
            return True
        if config.get("mailto"):
            config["link"] = {"external_link": f"mailto:{config.get('mailto')}"}
            del config["mailto"]
            return True
        if config.get("anchor"):
            config["link"] = {"external_link": "#" + config.get('anchor')}
            del config["anchor"]
            return True

    elif direction == "backward" and config.get("link"):
        link = config.get("link")
        if link.get("external_link"):
            if "#" in link.get("external_link"):
                config["anchor"] = link.get("external_link").split("#", 1)[1]
            ext = link.get("external_link").split("#", 1)[0]
            if ext.startswith("tel:"):
                config["phone"] = ext[4:]
            elif ext.startswith("mailto:"):
                config["mailto"] = ext[7:]
            elif ext:
                config["external_link"] = ext
        elif link.get("internal_link"):
            model, pk = link.get("internal_link").split(":")
            config["internal_link"] = {"model": model, "pk": int(pk)}
            if link.get("anchor"):
                config["anchor"] = link.get("anchor")[1:]
        elif link.get("file_link"):
            config["file_link"] = {"model": "filer.file", "pk": int(link.get("file_link"))}
        del config["link"]
        return True
    return False


def convert(apps, schema_editor, direction):
    FrontendUIItem = apps.get_model("djangocms_frontend", "FrontendUIItem")
    for item in FrontendUIItem.objects.all():
        changed = convert_item(item.config, direction)
        if changed:
            item.save()


class Migration(migrations.Migration):
    dependencies = [
        ("djangocms_frontend", "0001_initial"),
    ]

    operations = [
        migrations.RunPython(
            lambda apps, schema_editor: convert(apps, schema_editor, "forward"),
            lambda apps, schema_editor: convert(apps, schema_editor, "backward"),
            elidable=True
        ),
    ]



import importlib
import os
import warnings
from collections import defaultdict
from collections.abc import Iterator

from django import forms
from django.apps import apps
from django.template.loader import get_template
from django.utils.module_loading import autodiscover_modules
from sekizai.context import SekizaiContext

from djangocms_frontend import settings
from djangocms_frontend.component_base import CMSFrontendComponent


def find_cms_component_templates(subfolder: str) -> list[tuple[str, str]]:
    templates = []
    for app in apps.get_app_configs():
        app_template_dir = os.path.join(app.path, "templates", app.label, subfolder)
        if os.path.exists(app_template_dir):
            for root, _, files in os.walk(app_template_dir):
                for file in files:
                    if file.endswith(".html") or file.endswith(".htm"):
                        relative_path = os.path.relpath(os.path.join(root, file), app_template_dir)
                        templates.append((app.module.__name__, f"{app.label}/{subfolder}/{relative_path}"))
    return templates


class CMSAutoComponentDiscovery:
    default_field_context = {
        "djangocms_text": "djangocms_text.fields.HTMLFormField",
        "djangocms_text_ckeditor": "djangocms_text_ckeditor.fields.HTMLFormField",
        "djangocms_link": "djangocms_link.fields.LinkFormField",
        "djangocms_frontend": [
            "djangocms_frontend.contrib.image.fields.ImageFormField",
            "djangocms_frontend.contrib.icon.fields.IconPickerField",
            "djangocms_frontend.fields.AttributesFormField",
            "djangocms_frontend.fields.ChoicesFormField",
        ],
    }

    def __init__(self, register_to):
        self.default_field_context.update(settings.COMPONENT_FIELDS)
        templates = find_cms_component_templates(settings.COMPONENT_FOLDER)
        auto_components = self.scan_templates_for_component_declaration(templates)
        for component in auto_components:
            register_to.register(component)

    def get_field_context(self) -> dict:
        field_context = {}
        for key, value in self.default_field_context.items():
            if apps.is_installed(key):
                if not isinstance(value, list):
                    value = [value]
                for field in value:
                    if "." in field:
                        module, field_name = field.rsplit(".", 1)
                        field_context[field_name] = importlib.import_module(module).__dict__[field_name]
            elif key not in ("djangocms_text", "djangocms_text_ckeditor"):
                warnings.warn(f"App {key} not installed, skipping field registration", stacklevel=2)
        return field_context

    @staticmethod
    def component_factory(module, component: tuple, fields: list[tuple], template: str) -> CMSFrontendComponent:
        args, kwargs = component
        (name,) = args

        kwargs["render_template"] = template
        meta = type("Meta", (), kwargs)
        return type(
            name,
            (CMSFrontendComponent,),
            {
                "Meta": meta,
                "__module__": module,
                **{
                    # Django template engine instantiates objects -- re-instantiate them here
                    args[0]: args[1].__class__(**kwargs)
                    for args, kwargs in fields
                    if isinstance(args[1], forms.Field)
                },
            },
        )

    def scan_templates_for_component_declaration(
        self, templates: list[tuple[str, str]]
    ) -> Iterator[CMSFrontendComponent]:
        from django.forms import fields

        field_context = self.get_field_context()
        for module, template_name in templates:
            # Create a new context for each template
            context = SekizaiContext(
                {"_cms_components": defaultdict(list), "forms": fields, "instance": {}, **field_context}
            )
            try:
                template = get_template(template_name)
                template.template.render(context)
                cms_component = context["_cms_components"].get("cms_component", [])
                discovered_fields = context["_cms_components"].get("fields", [])
                if len(cms_component) == 1:
                    yield self.component_factory(module, cms_component[0], discovered_fields, template_name)
                elif len(cms_component) > 1:  # pragma: no cover
                    raise ValueError(f"Multiple cms_component tags found in {template_name}")
            except Exception:  # pragma: no cover
                # Skip all templates that do not render
                import logging

                logger = logging.getLogger(__name__)
                logger.error(
                    f"Error rendering template {template_name} to scan for cms frontend components", exc_info=True
                )


class Components:
    _registry: dict = {}
    _discovered: bool = False

    def register(self, component):
        if component.__name__ in self._registry:  # pragma: no cover
            warnings.warn(f"Component {component.__name__} already registered", stacklevel=2)
            return component
        self._registry[component.__name__] = component.get_registration()
        return component

    def __getitem__(self, item):
        return self._registry[item]


components = Components()


def setup():
    if not components._discovered:
        from .cms_plugins import update_plugin_pool

        # First discover components in cms_components module
        autodiscover_modules("cms_components", register_to=components)
        # The discover auto components by their templates
        CMSAutoComponentDiscovery(register_to=components)
        update_plugin_pool()
        components._discovered = True

        if apps.is_installed("djangocms_text"):
            # Hack - update inline editable fields in case djangocms_text is installed
            # BEFORE djangocms_frontend in INSTALLED_APPS:
            # Need to initialize inline fields again to reflect inline fields of the just discovered components
            from djangocms_text.apps import discover_inline_editable_models

            text_config = apps.get_app_config("djangocms_text")
            text_config.inline_models = discover_inline_editable_models()



import copy
import decimal

from cms.constants import SLUG_REGEXP
from cms.plugin_base import CMSPluginBase
from cms.utils.conf import get_cms_setting
from django.apps import apps
from django.contrib.admin.helpers import AdminForm
from django.db.models import ObjectDoesNotExist
from django.shortcuts import render
from django.template.exceptions import TemplateDoesNotExist
from django.template.loader import select_template
from django.urls import re_path
from django.utils.functional import lazy
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from djangocms_frontend import settings
from djangocms_frontend.settings import FRAMEWORK_PLUGIN_INFO


def get_related_object(scope, field_name):
    """
    Returns the related field, referenced by the content of a ModelChoiceField.
    """
    try:
        Model = apps.get_model(scope[field_name]["model"])
        relobj = Model.objects.get(pk=scope[field_name]["pk"])
    except (ObjectDoesNotExist, LookupError, TypeError):
        relobj = None
    return relobj


class get_related:
    """Descriptor lazily getting related objects from the config dict."""

    def __init__(self, key):
        self.key = key

    def __get__(self, instance, owner):
        obj = get_related_object(instance.config, self.key)
        if obj is not None:
            setattr(instance, self.key, obj)
        return obj


def insert_fields(fieldsets, new_fields, block=None, position=-1, blockname=None, blockattrs=None):
    """
    creates a copy of fieldsets inserting the new fields either in the indexed block at the position,
    or - if no block is given - at the end
    """
    if blockattrs is None:
        blockattrs = dict()
    if block is None:
        fs = (
            list(fieldsets[:position] if position != -1 else fieldsets)
            + [
                (
                    blockname,
                    {
                        "classes": ("collapse",) if len(fieldsets) > 0 else (),
                        "fields": list(new_fields),
                        **blockattrs,
                    },
                )
            ]
            + list(fieldsets[position:] if position != -1 else [])
        )
        return fs
    modify = copy.deepcopy(fieldsets[block])
    fields = modify[1]["fields"]
    if position >= 0:
        modify[1]["fields"] = list(fields[:position]) + list(new_fields) + list(fields[position:])
    else:
        modify[1]["fields"] = (
            list(fields[: position + 1] if position != -1 else fields)
            + list(new_fields)
            + list(fields[position + 1 :] if position != -1 else [])
        )
    fs = (
        list(fieldsets[:block] if block != -1 else fieldsets)
        + [modify]
        + list(fieldsets[block + 1 :] if block != -1 else [])
    )
    return fs


def first_choice(choices):
    for value, verbose in choices:
        if not isinstance(verbose, (tuple, list)):
            return value
        else:
            first = first_choice(verbose)
            if first is not None:
                return first
    return None


def get_template_path(prefix, template, name):
    return f"djangocms_frontend/{settings.framework}/{prefix}/{template}/{name}.html"


def get_plugin_template(instance, prefix, name, templates):
    template = getattr(instance, "template", first_choice(templates))
    template_path = get_template_path(prefix, template, name)

    try:
        select_template([template_path])
    except TemplateDoesNotExist:
        # TODO render a warning inside the template
        template_path = get_template_path(prefix, "default", name)

    return template_path


# use mark_safe_lazy to delay the translation when using mark_safe
# otherwise they will not be added to /locale/
# https://docs.djangoproject.com/en/1.11/topics/i18n/translation/#other-uses-of-lazy-in-delayed-translations
mark_safe_lazy = lazy(mark_safe, str)


def link_to_framework_doc(ui_item, topic):
    link = FRAMEWORK_PLUGIN_INFO.get(ui_item, {}).get(topic, None)
    if link:
        return mark_safe_lazy(
            _('Read more in the <a href="{link}" target="_blank">documentation</a>.').format(link=link)
        )
    return None


def add_plugin(placeholder, plugin):
    """CMS version-save function to add a plugin to a placeholder"""
    if hasattr(placeholder, "add_plugin"):  # available as of CMS v4
        placeholder.add_plugin(plugin)
    else:  # CMS < v4
        if plugin.parent:
            plugin.position -= plugin.parent.position + 1  # Restart position counting at 0
        else:
            plugin.position -= 1  # 0-based counting in v3
        plugin.save()


def delete_plugin(plugin):
    """CMS version save function to delete a plugin (and its descendants) from a placeholder"""
    return plugin.placeholder.delete_plugin(plugin)


def is_first_child(instance, parent):
    if hasattr(instance.placeholder, "add_plugin"):  # available as of CMS v4
        return instance.position == parent.position + 1
    else:
        return instance.position == 0


def coerce_decimal(value):
    """Force value to be converted to decimal.Decimal or return None"""
    try:
        return decimal.Decimal(value)
    except TypeError:
        return None


class FrontendEditableAdminMixin:
    """
    Adding ``FrontendEditableAdminMixin`` to  models admin class allows to open that admin
    in the frontend by double-clicking on fields rendered with the ``render_model`` template
    tag.
    """

    frontend_editable_fields = []

    def get_urls(self):  # pragma: no cover
        """
        Register the url for the single field edit view
        """
        info = f"{self.model._meta.app_label}_{self.model._meta.model_name}"

        def pat(regex, fn):
            return re_path(regex, self.admin_site.admin_view(fn), name=f"{info}_{fn.__name__}")

        url_patterns = [
            pat(r"edit-field/(%s)/([a-z\-]+)/$" % SLUG_REGEXP, self.edit_field),
        ]
        return url_patterns + super().get_urls()

    def _get_object_for_single_field(self, object_id, language):  # pragma: no cover
        # Quick and dirty way to retrieve objects for django-hvad
        # Cleaner implementation will extend this method in a child mixin
        try:
            # First see if the model uses the admin manager pattern from cms.models.manager.ContentAdminManager
            manager = self.model.admin_manager
        except AttributeError:
            # If not, use the default manager
            manager = self.model.objects
        try:
            return manager.language(language).get(pk=object_id)
        except AttributeError:
            return manager.get(pk=object_id)

    def edit_field(self, request, object_id, language):
        obj = self._get_object_for_single_field(object_id, language)
        opts = obj.__class__._meta
        saved_successfully = False
        cancel_clicked = request.POST.get("_cancel", False)
        raw_fields = request.GET.get("edit_fields")
        fields = [field for field in raw_fields.split(",") if field in self.frontend_editable_fields]
        if not fields:
            context = {"opts": opts, "message": _("Field %s not found") % raw_fields}
            return render(request, "admin/cms/page/plugin/error_form.html", context)
        if not request.user.has_perm(f"{self.model._meta.app_label}.change_{self.model._meta.model_name}"):
            context = {"opts": opts, "message": _("You do not have permission to edit this item")}
            return render(request, "admin/cms/page/plugin/error_form.html", context)
            # Dynamically creates the form class with only `field_name` field
        # enabled
        form_class = self.get_form(request, obj, fields=fields)
        if not cancel_clicked and request.method == "POST":
            form = form_class(instance=obj, data=request.POST)
            if form.is_valid():
                new_object = form.save(commit=False)
                self.save_model(request, new_object, form, change=True)
                saved_successfully = True
        else:
            form = form_class(instance=obj)
        admin_form = AdminForm(form, fieldsets=[(None, {"fields": fields})], prepopulated_fields={}, model_admin=self)
        media = self.media + admin_form.media
        context = {
            "CMS_MEDIA_URL": get_cms_setting("MEDIA_URL"),
            "title": opts.verbose_name,
            "plugin": None,
            "plugin_id": None,
            "adminform": admin_form,
            "add": False,
            "is_popup": True,
            "media": media,
            "opts": opts,
            "change": True,
            "save_as": False,
            "has_add_permission": False,
            "window_close_timeout": 10,
        }
        if cancel_clicked:
            # cancel button was clicked
            context.update(
                {
                    "cancel": True,
                }
            )
            return render(request, "admin/cms/page/plugin/confirm_form.html", context)
        if not cancel_clicked and request.method == "POST" and saved_successfully:
            if isinstance(self, CMSPluginBase):
                if hasattr(obj.placeholder, "mark_as_dirty"):
                    # Only relevant for v3: mark the placeholder as dirty so user can publish changes
                    obj.placeholder.mark_as_dirty(obj.language, clear_cache=False)
                # Update the structure board by populating the data bridge
                return self.render_close_frame(request, obj)
            return render(request, "admin/cms/page/plugin/confirm_form.html", context)
        return render(request, "admin/cms/page/plugin/change_form.html", context)



from cms.plugin_pool import plugin_pool
from django.apps import apps
from django.conf import settings as django_settings
from django.utils.translation import gettext_lazy as _

from djangocms_frontend.helpers import get_plugin_template, insert_fields

from ... import settings
from ...cms_plugins import CMSUIPlugin
from ...common import AttributesMixin, SpacingMixin
from .. import link
from . import forms, models
from .constants import USE_LINK_ICONS
from .helpers import GetLinkMixin

mixin_factory = settings.get_renderer(link)


UILINK_FIELDS = (
    ("name", "link_type"),
    ("site", "url_grouper") if apps.is_installed("djangocms_url_manager") else "link",
    ("link_context", "link_size"),
    ("link_outline", "link_block"),
    "link_stretched",
)

UILINK_FIELDSET = [
    (
        None,
        {
            "fields": ("template",)
            + (UILINK_FIELDS + (("icon_left", "icon_right"),) if USE_LINK_ICONS else UILINK_FIELDS)
        },
    ),
]


class LinkPluginMixin:
    link_fieldset_position = None
    link_fields = (("site", "url_grouper"),) if apps.is_installed("djangocms_url_manager") else ("link", "target")

    def render(self, context, instance, placeholder):
        if "request" in context:
            instance._cms_page = getattr(context["request"], "current_page", None)
        context["mixin_link"] = instance.get_link()
        return super().render(context, instance, placeholder)

    def get_form(self, request, obj=None, change=False, **kwargs):
        """The link form needs the request object to check permissions"""
        form = super().get_form(request, obj, change, **kwargs)
        form.request = request
        parent = obj.parent if obj else self._cms_initial_attributes.get("parent")
        form.name_required = parent and parent.plugin_type == "TextPlugin"
        return form

    def get_fieldsets(self, request, obj=None):
        fieldsets = super().get_fieldsets(request, obj)
        if self.link_fieldset_position is not None:
            fieldsets = insert_fields(
                fieldsets,
                self.link_fields,
                blockname=_("Link settings"),
                position=self.link_fieldset_position,
            )
        return fieldsets


class TextLinkPlugin(mixin_factory("Link"), AttributesMixin, SpacingMixin, LinkPluginMixin, GetLinkMixin, CMSUIPlugin):
    """
    Components > "Button" Plugin
    https://getbootstrap.com/docs/5.0/components/buttons/
    """

    name = _("Link / Button")
    module = _("Frontend")
    model = models.Link
    form = forms.LinkForm
    change_form_template = "djangocms_frontend/admin/link.html"
    text_enabled = True
    text_icon = (
        '<svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" fill="currentColor" class="bi bi-link-45deg" '
        'viewBox="0 0 16 16"><path d="M4.715 6.542 3.343 7.914a3 3 0 1 0 4.243 4.243l1.828-1.829A3 3 0 0 0 8.586 '
        "5.5L8 6.086a1 1 0 0 0-.154.199 2 2 0 0 1 .861 3.337L6.88 11.45a2 2 0 1 1-2.83-2.83l.793-.792a4 4 0 0 "
        '1-.128-1.287z"/><path d="M6.586 4.672A3 3 0 0 0 7.414 9.5l.775-.776a2 2 0 0 1-.896-3.346L9.12 3.55a2 2 0 '
        '1 1 2.83 2.83l-.793.792c.112.42.155.855.128 1.287l1.372-1.372a3 3 0 1 0-4.243-4.243z"/></svg>'
    )
    allow_children = True

    fieldsets = UILINK_FIELDSET

    def get_render_template(self, context, instance, placeholder):
        return get_plugin_template(instance, "link", "link", settings.LINK_TEMPLATE_CHOICES)


if "djangocms_frontend.contrib.link" in django_settings.INSTALLED_APPS:
    #  Only register plugin if in INSTALLED_APPS
    plugin_pool.register_plugin(TextLinkPlugin)

    if "djangocms_link" in django_settings.INSTALLED_APPS:
        from djangocms_link.cms_plugins import LinkPlugin

        LinkPlugin.parent_classes = [""]  # Remove it from the list of valid plugins



from django import apps, forms
from django.conf import settings as django_settings
from django.contrib.sites.models import Site
from django.utils.translation import gettext as _
from djangocms_link.fields import LinkFormField

# from djangocms_link.validators import IntranetURLValidator
from entangled.forms import EntangledModelForm, EntangledModelFormMixin

from ... import settings
from ...common import SpacingFormMixin
from ...fields import AttributesFormField, ButtonGroup, ColoredButtonGroup, TagTypeFormField, TemplateChoiceMixin
from ...helpers import first_choice
from ...models import FrontendUIItem
from .. import link
from .constants import LINK_CHOICES, LINK_SIZE_CHOICES, TARGET_CHOICES

mixin_factory = settings.get_forms(link)

if "djangocms_frontend.contrib.icon" in django_settings.INSTALLED_APPS:
    # Weak dependency on djangocms_frontend.contrib.icon
    from djangocms_frontend.contrib.icon.fields import IconPickerField
elif "djangocms_icon" in django_settings.INSTALLED_APPS:  # pragma: no cover
    # Weak dependency on djangocms_icon
    # (Even if djangocms_icon is in the python path, the admin form will fail due to missing
    # templates if it's not in INSTALLED_APPS)
    from djangocms_icon.fields import IconField as IconPickerField
else:  # pragma: no cover

    class IconPickerField(forms.CharField):  # lgtm [py/missing-call-to-init]
        def __init__(self, *args, **kwargs):
            kwargs["widget"] = forms.HiddenInput
            super().__init__(*args, **kwargs)


if apps.apps.is_installed("djangocms_url_manager"):
    from djangocms_url_manager.forms import HtmlLinkSiteSelectWidget, HtmlLinkUrlSelectWidget
    from djangocms_url_manager.models import UrlGrouper

    class LinkFormMixin(EntangledModelFormMixin):
        class Meta:
            entangled_fields = {
                "config": [
                    "site",
                    "url_grouper",
                ]
            }

        site = forms.ModelChoiceField(
            label=_("Site"),
            queryset=Site.objects.all(),
            widget=HtmlLinkSiteSelectWidget(attrs={"data-placeholder": _("Select site")}),
            required=False,
        )
        url_grouper = forms.ModelChoiceField(
            label=_("Url"),
            queryset=UrlGrouper.objects.all(),
            widget=HtmlLinkUrlSelectWidget(attrs={"data-placeholder": _("Select URL object from list")}),
            required=False,
        )

else:

    class LinkFormMixin(EntangledModelFormMixin):
        class Meta:
            entangled_fields = {
                "config": [
                    "link",
                    "target",
                ]
            }

        link_is_optional = False

        link = LinkFormField(
            label=_("Link"),
            required=False,
        )
        target = forms.ChoiceField(
            label=_("Target"),
            choices=settings.EMPTY_CHOICE + TARGET_CHOICES,
            required=False,
        )

        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.fields["link"].required = not self.link_is_optional


class AbstractLinkForm(LinkFormMixin, EntangledModelForm):
    pass


class LinkForm(mixin_factory("Link"), SpacingFormMixin, TemplateChoiceMixin, AbstractLinkForm):
    class Meta:
        model = FrontendUIItem
        entangled_fields = {
            "config": [
                "name",
                "template",
                "link_type",
                "link_context",
                "link_size",
                "link_outline",
                "link_block",
                "icon_left",
                "icon_right",
                "link_stretched",
                "attributes",
            ]
        }
        untangled_fields = ()
        exclude = ("ui_item",)

    name = forms.CharField(
        label=_("Display name"),
        required=False,
        widget=forms.TextInput(attrs={"class": "js-prepopulate-selected-text"}),
    )
    template = forms.ChoiceField(
        label=_("Layout"),
        choices=settings.LINK_TEMPLATE_CHOICES,
        initial=first_choice(settings.LINK_TEMPLATE_CHOICES),
    )
    link_stretched = forms.BooleanField(
        label=_("Stretch link"),
        required=False,
        initial=False,
        help_text=_("Stretches the active link area to the containing block (with position: relative)."),
    )
    link_type = forms.ChoiceField(
        label=_("Type"),
        choices=LINK_CHOICES,
        initial=first_choice(LINK_CHOICES),
        widget=forms.RadioSelect(attrs={"class": "inline-block"}),
        help_text=_("Adds either a text link or a button which links to the target."),
    )
    link_context = forms.ChoiceField(
        label=_("Context"),
        choices=settings.EMPTY_CHOICE + settings.COLOR_STYLE_CHOICES,
        initial=settings.EMPTY_CHOICE[0][0],
        required=False,
        widget=ColoredButtonGroup(),
    )
    link_size = forms.ChoiceField(
        label=_("Button size"),
        choices=LINK_SIZE_CHOICES,
        initial=LINK_SIZE_CHOICES[1][0],  # Medium
        required=False,
        widget=ButtonGroup(attrs=dict(property="link-size", label_class="btn-secondary")),
    )
    link_outline = forms.BooleanField(
        label=_("Outline"),
        initial=False,
        required=False,
        help_text=_("Removes the coloring from a button and keeps the outline."),
    )
    link_block = forms.BooleanField(
        label=_("Block"),
        initial=False,
        required=False,
        help_text=_("Extends the button to the width of its container."),
    )
    icon_left = IconPickerField(
        label=_("Icon left"),
        initial="",
        required=False,
    )
    icon_right = IconPickerField(
        label=_("Icon right"),
        initial="",
        required=False,
    )
    attributes = AttributesFormField()
    tag_type = TagTypeFormField()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # name_required is set by the plugin's get_form method if needed
        self.fields["name"].required = getattr(self, "name_required", False)



from cms.plugin_pool import plugin_pool
from django.utils.translation import gettext_lazy as _

from ... import settings
from ...cms_plugins import CMSUIPlugin
from ...common import AttributesMixin, BackgroundMixin
from ...helpers import first_choice, get_plugin_template, get_template_path
from .. import navigation
from ..link.cms_plugins import LinkPluginMixin, TextLinkPlugin
from . import forms, models

mixin_factory = settings.get_renderer(navigation)

default_template = first_choice(settings.NAVIGATION_TEMPLATE_CHOICES)


@plugin_pool.register_plugin
class NavigationPlugin(
    mixin_factory("Navigation"),
    AttributesMixin,
    BackgroundMixin,
    CMSUIPlugin,
):
    """
    The NavigationPlugin class is a plugin used in Django CMS to create navigation menus or off-canvas menus.
    """

    name = _("Navigation")
    module = _("Frontend")
    model = models.Navigation
    form = forms.NavigationForm
    change_form_template = "djangocms_frontend/admin/navigation.html"
    allow_children = True
    child_classes = [
        "NavLinkPlugin",
        "PageTreePlugin",
        "NavBrandPlugin",
    ]

    fieldsets = [
        (
            None,
            {
                "fields": (
                    "template",
                    (
                        "navbar_design",
                        "navbar_breakpoint",
                    ),
                    "navbar_container",
                )
            },
        ),
    ]

    def get_render_template(self, context, instance, placeholder):
        return get_plugin_template(instance, "navigation", "navigation", settings.NAVIGATION_TEMPLATE_CHOICES)

    def render(self, context, instance, placeholder):
        context["nav_template"] = instance.config.get("template", default_template)
        return super().render(context, instance, placeholder)


@plugin_pool.register_plugin
class PageTreePlugin(
    mixin_factory("PageTree"),
    AttributesMixin,
    CMSUIPlugin,
):
    """

    The PageTreePlugin class is a plugin for Django CMS that allows users to display a hierarchical
    tree-like structure of pages on the frontend.
    """

    name = _("Page tree")
    module = _("Frontend")
    model = models.PageTree
    form = forms.PageTreeForm
    change_form_template = "djangocms_frontend/admin/page_tree.html"
    allow_children = False
    parent_classes = [
        "NavigationPlugin",
    ]
    show_add_form = False
    fieldsets = [
        (
            None,
            {"fields": ("start_level",)},
        ),
    ]

    def get_render_template(self, context, instance, placeholder):
        return get_template_path("navigation", context.get("nav_template", default_template), "page_tree")

    def render(self, context, instance, placeholder):
        context["start_level"] = instance.config.get("start_level", 0)
        return super().render(context, instance, placeholder)


@plugin_pool.register_plugin
class NavBrandPlugin(
    mixin_factory("NavBrand"),
    AttributesMixin,
    LinkPluginMixin,
    CMSUIPlugin,
):
    """
    The `NavBrandPlugin` class is a plugin used in Django CMS to create a navigation brand element.
    This plugin allows the user to define a brand logo or text that will be displayed in the
    navigation header. Content is added through child plugins.
    """

    name = _("Brand")
    module = _("Frontend")
    model = models.NavBrand
    form = forms.NavBrandForm
    change_form_template = "djangocms_frontend/admin/brand.html"
    allow_children = True
    parent_classes = [
        "NavigationPlugin",
    ]
    link_fieldset_position = -1

    fieldsets = [
        (
            None,
            {"fields": ("simple_content",)},
        ),
    ]

    def get_render_template(self, context, instance, placeholder):
        return get_template_path("navigation", context.get("nav_template", default_template), "brand")


@plugin_pool.register_plugin
class NavContainerPlugin(
    mixin_factory("NavLink"),
    AttributesMixin,
    CMSUIPlugin,
):
    """
    The `NavContainerPlugin` class is a deprecated plugin without functionality. It will be removed.
    """

    name = _("Navigation container")
    module = _("Frontend")
    model = models.NavContainer
    change_form_template = "djangocms_frontend/admin/deprecated.html"
    allow_children = True
    parent_classes = [""]  # No parent classes
    child_classes = [
        "NavLinkPlugin",
        "PageTreePlugin",
        "NavBrandPlugin",
    ]

    fieldsets = [
        (
            None,
            {"fields": ()},
        ),
    ]


@plugin_pool.register_plugin
class NavLinkPlugin(
    mixin_factory("NavLink"),
    TextLinkPlugin,
):
    """
    A plugin that allows creating navigation links for the frontend.

    Attributes:
    -----------
    - `name` (str): The name of the plugin, displayed in the plugin list when editing a page.
    - `module` (str): The module where the plugin belongs, displayed in the plugin list when editing a page.
    - `model` (Model): The Django model used to store the plugin's data.
    - `form` (Form): The form used to render the plugin's settings in the admin interface.
    - `change_form_template` (str): The path to the template used to render the plugin's change form in the admin
      interface.
    - `allow_children` (bool): Whether the plugin allows having child plugins.
    - `parent_classes` (list): List of parent plugin classes that this plugin can be nested within.
    - `child_classes` (list): List of child plugin classes that can be nested within this plugin.
    """

    name = _("Navigation link")
    module = _("Frontend")
    model = models.NavLink
    form = forms.NavLinkForm
    change_form_template = "djangocms_frontend/admin/navlink.html"
    allow_children = True
    parent_classes = ["NavigationPlugin", "NavLinkPlugin"]
    child_classes = [
        "NavLinkPlugin",
        "GridContainerPlugin",
        "GridRowPlugin",
        "ListGroupPlugin",
    ]

    def get_render_template(self, context, instance, placeholder):
        return get_template_path("navigation", context.get("nav_template", default_template), "link")



from cms.test_utils.testcases import CMSTestCase
from django.contrib.sites.shortcuts import get_current_site
from django.template import engines
from django.test import override_settings

from tests.fixtures import TestFixture

django_engine = engines["django"]


@override_settings(DEBUG=True)
class PluginTagTestCase(TestFixture, CMSTestCase):
    def test_tag_default_rendering(self):
        template = django_engine.from_string("""
        {% load frontend cms_tags %}
        {% plugin "alert" %}Alert{% endplugin %}
        """)

        result = template.render({"request": None})

        self.assertInHTML('<div class="alert-primary alert" role="alert"><div>Alert</div></div>', result)

    def test_tag_rendering_with_paramter(self):
        template = django_engine.from_string("""
        {% load frontend cms_tags %}
        {% plugin "alert" alert_context="secondary" alert_dismissible=True %}Alert{% endplugin %}
        """)
        expected_result = """<div class="alert alert-secondary alert-dismissible" role="alert">
        <button type="button" class="btn-close" data-bs-dismiss="alert" aria-label="Close"></button><div>Alert
        </div></div>"""

        result = template.render({"request": None})

        self.assertInHTML(expected_result, result)

    def test_simple_tag(self):
        template = django_engine.from_string("""
        {% load frontend  %}
        {% plugin "badge" badge_text="My badge" badge_context="info" badge_pills=False %}
           This content is ignored.
        {% endplugin %}""")
        expected_result = """<span class="badge bg-info">My badge</span>"""

        result = template.render({"request": None})

        self.assertInHTML(expected_result, result)

    def test_complex_tags(self):
        from djangocms_frontend.plugin_tag import plugin_tag_pool

        template = django_engine.from_string("""{% load frontend %}
        {% plugin "card" card_alignment="center" card_outline="info" card_text_color="primary" card_full_height=True %}
            {% plugin "cardinner" inner_type="card-header" text_alignment="start" %}
                <h4>Card title</h4>
            {% endplugin %}
            {% plugin "cardinner" inner_type="card-body" text_alignment="center" %}
                Some quick example text to build on the card title and make up the
                bulk of the card's content.
            {% endplugin %}
            {% plugin "listgroupitem" %}An item{% endplugin %}
            {% plugin "listgroupitem" %}A second item{% endplugin %}
            {% plugin "listgroupitem" %}A third item{% endplugin %}
        {% endplugin %}""")

        expected_result = """
        <div class="card text-primary text-center border-info h-100">
            <div class="text-start card-header"><h4>Card title</h4></div>
            <div class="card-body text-center">
                Some quick example text to build on the card title and make up the
                bulk of the card's content.
            </div>
            <div class="list-group-item">An item</div>
            <div class="list-group-item">A second item</div>
            <div class="list-group-item">A third item</div>
        </div>"""

        result = template.render({"request": None})

        self.assertIn("cardinner", plugin_tag_pool)

        self.assertInHTML(expected_result, result)

    def test_link_plugin(self):
        if hasattr(self, "create_url"):
            grouper = self.create_url(manual_url="/test/").url_grouper
            template = django_engine.from_string("""{% load frontend %}
            {% plugin "textlink" name="Click" url_grouper=grouper site=test_site link_type="btn" link_context="primary" link_outline=False %}
                Click me!
            {% endplugin %}
        """)  # noqa: B950,E501
        else:
            grouper = None
            template = django_engine.from_string("""{% load frontend djangocms_link_tags %}{{ "test"|to_link }}
                {% plugin "textlink" name="Click" link="/test/"|to_link link_type="btn" link_context="primary" link_outline=False %}
                    Click me!
                {% endplugin %}
            """)  # noqa: B950,E501

        expected_result = """<a href="/test/" class="btn btn-primary">Click me!</a>"""

        result = template.render({"request": None, "test_site": get_current_site(None), "grouper": grouper})
        self.assertInHTML(expected_result, result)

    @override_settings(DEBUG=True)
    def test_non_existing_plugin(self):
        template = django_engine.from_string("""{% load frontend %}
            {% plugin "nonexisting" %}
                This should not be rendered.
            {% endplugin %}
        """)
        expected_result = (
            '<!-- To use "nonexisting" with the {% plugin %} template tag, add its plugin class to the '
            "CMS_COMPONENT_PLUGINS setting -->"
        )

        result = template.render({"request": None})

        self.assertEqual(expected_result.strip(), result.strip())

    def test_non_frontend_plugin(self):
        template = django_engine.from_string("""{% load frontend %}
            {% plugin "text" body="<p>my text</p>" %}
                This should not be rendered.
            {% endplugin %}
        """)
        expected_result = "<p>my text</p>"

        result = template.render({"request": None})

        self.assertInHTML(expected_result, result)

    def test_autohero_component_registered_for_plugin_tag(self):
        from cms.plugin_pool import plugin_pool

        from djangocms_frontend.plugin_tag import plugin_tag_pool

        # Check that the AutoHero plugin is registered
        self.assertIn("AutoHeroPlugin", plugin_pool.plugins)

        # Check for the AutoHero plugin registration in the plugin_tag_pool
        self.assertIn("autohero", plugin_tag_pool)



from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_frontend.contrib.navigation.cms_plugins import (
    NavigationPlugin,
    NavLinkPlugin,
    PageTreePlugin,
)
from djangocms_frontend.contrib.navigation.forms import (
    NavigationForm,
    NavLinkForm,
    PageTreeForm,
)

from ..fixtures import TestFixture


class NavigationPluginTestCase(TestFixture, CMSTestCase):
    def test_plugin(self):
        nav = add_plugin(
            placeholder=self.placeholder,
            plugin_type=NavigationPlugin.__name__,
            language=self.language,
            config=dict(
                navbar_breakpoint="xl",
                navbar_container=True,
            ),
        ).initialize_from_form(NavigationForm)
        nav.save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(
            '<nav class="navbar-expand-xl navbar navbar-light">' in response.content.decode("utf-8")
            or '<nav class="navbar-expand-xl navbar-light navbar">' in response.content.decode("utf-8")
            or '<nav class="navbar navbar-light navbar-expand-xl">' in response.content.decode("utf-8")
            or '<nav class="navbar navbar-expand-xl navbar-light">' in response.content.decode("utf-8")
            or '<nav class="navbar-light navbar navbar-expand-xl">' in response.content.decode("utf-8")
            or '<nav class="navbar-light navbar-expand-xl navbar">' in response.content.decode("utf-8")
        )
        self.assertContains(response, '<div class="container">')
        self.assertContains(response, '<span class="navbar-toggler-icon"></span>')
        self.assertContains(response, '<div class="collapse navbar-collapse"')
        self.assertContains(response, '<ul class="navbar-nav">')

        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=NavLinkPlugin.__name__,
            language=self.language,
            target=nav,
            config=dict(
                link=dict(internal_link=f"cms.page:{self.page.id}"),
                link_context="primary",
                link_type="btn",
                name="django CMS rocks!",
            ),
        )
        plugin.initialize_from_form(NavLinkForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "btn-primary")
        self.assertContains(response, "nav-link")
        self.assertContains(response, 'href="/content/"')

        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=PageTreePlugin.__name__,
            language=self.language,
            target=nav,
            config=dict(),
        )
        plugin.initialize_from_form(PageTreeForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)



from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_frontend.contrib.image.models import Image
from tests.fixtures import TestFixture


class DjangoCMSPictureIntegrationTestCase(TestFixture, CMSTestCase):
    def setUp(self):
        super().setUp()
        self.placeholder = self.get_placeholders(self.home).get(slot="content")

    def test_extract_images(self):
        text_plugin = add_plugin(
            self.placeholder,
            "TextPlugin",
            "en",
            body='<img src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAADUlEQVR42m'
            'P8z/C/HgAGgwJ/lK3Q6wAAAABJRU5ErkJggg==">',
        )

        picture_plugins = Image.objects.order_by("-id")
        self.assertEqual(len(picture_plugins), 1)
        self.assertEqual(picture_plugins[0].parent.id, text_plugin.id)
        id = picture_plugins[0].id
        self.assertHTMLEqual(
            text_plugin.body,
            f'<cms-plugin alt="Picture / Image - Image ({id})" '
            f'title="Picture / Image - Image ({id})" '
            f'id="{id}"></cms-plugin>',
        )



from __future__ import annotations

from cms.plugin_pool import plugin_pool

from .ui_plugin_base import CMSUIPluginBase


class CMSUIPlugin(CMSUIPluginBase):
    pass


def update_plugin_pool():
    from .component_pool import components

    # Loop through the values in the components' registry
    for _, plugin, slot_plugins in components._registry.values():
        if plugin.__name__ not in plugin_pool.plugins:
            # Add the plugin to the global namespace
            globals()[plugin.__name__] = plugin
            # Register the plugin with the plugin pool
            plugin_pool.register_plugin(plugin)

            # Loop through the slot plugins associated with the current plugin
            for slot_plugin in slot_plugins:
                # Add the slot plugin to the global namespace
                # Register slot plugins, checking for name conflicts
                for slot_plugin in slot_plugins:
                    if slot_plugin.__name__ not in plugin_pool.plugins:
                        globals()[slot_plugin.__name__] = slot_plugin
                        plugin_pool.register_plugin(slot_plugin)



from unittest.mock import patch

from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase
from django.template import Context

from djangocms_frontend.contrib.collapse.cms_plugins import (
    CollapseContainerPlugin,
    CollapsePlugin,
    CollapseTriggerPlugin,
)
from djangocms_frontend.contrib.collapse.forms import (
    CollapseContainerForm,
    CollapseForm,
    CollapseTriggerForm,
)
from djangocms_frontend.models import FrontendUIItem
from djangocms_frontend.templatetags.frontend import set_html_id

from ..fixtures import TestFixture


class CollapsePluginTestCase(TestFixture, CMSTestCase):
    def test_collapse_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=CollapsePlugin.__name__,
            language=self.language,
        )
        plugin.initialize_from_form(CollapseForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'data-bs-children=".card"')
        self.assertContains(response, 'role="tablist"')

    def test_collapse_trigger_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=CollapseTriggerPlugin.__name__,
            language=self.language,
            config=dict(
                trigger_identifier=10,
            ),
        )
        plugin.initialize_from_form(CollapseTriggerForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'aria-controls="10"')
        self.assertContains(response, 'data-bs-target="#10"')
        self.assertContains(response, 'id="trigger-10"')

    def test_collapse_container_plugin(self):
        parent = add_plugin(
            placeholder=self.placeholder,
            plugin_type=CollapsePlugin.__name__,
            language=self.language,
        )
        parent.initialize_from_form(CollapseForm).save()

        plugin = add_plugin(
            target=parent,
            placeholder=self.placeholder,
            plugin_type=CollapseContainerPlugin.__name__,
            language=self.language,
            config=dict(
                container_identifier=10,
            ),
        )
        plugin.initialize_from_form(CollapseContainerForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'aria-labelledby="trigger-10"')
        self.assertContains(response, "10")

    def test_collapse_html_id(self):
        def create_plugin():
            plugin = add_plugin(
                placeholder=self.placeholder,
                plugin_type=CollapsePlugin.__name__,
                language=self.language,
            )
            plugin.initialize_from_form(CollapseForm).save()
            self.publish(self.page, self.language)

        create_plugin()
        create_plugin()
        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            """<div id="collapse-frontend-plugins-1" data-bs-children=".card" role="tablist"></div>""",
            html=True,
        )
        self.assertContains(
            response,
            """<div id="collapse-frontend-plugins-2" data-bs-children=".card" role="tablist"></div>""",
            html=True,
        )

    def test_set_html_id(self):
        instance = FrontendUIItem()
        context = Context()
        with patch("os.urandom", lambda n: b"\x1bB\x96\xabyI\xf6`\xd0\xc0,\xf8\x83\xe8,\xb8"):
            html_id = set_html_id(context, instance)
        identifier = "uuid4-1b4296ab-7949-4660-90c0-2cf883e82cb8"
        self.assertEqual(html_id, identifier)
        self.assertEqual(instance.html_id, identifier)



from django.apps import apps
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.core.exceptions import ObjectDoesNotExist

from djangocms_frontend.helpers import get_related_object


def get_object_for_value(value):
    if isinstance(value, str) and "-" in value:
        type_id, obj_id = value.split("-", 1)
        try:
            content_type = ContentType.objects.get(id=type_id)
            return dict(
                model=f"{content_type.app_label}.{content_type.model}",
                pk=int(obj_id),
            )
        except (ObjectDoesNotExist, TypeError):
            pass
    return None


class GetLinkMixin:
    def get_link(self) -> str:
        if "url_grouper" in self.config and self.config["url_grouper"] and apps.is_installed("djangocms_url_manager"):
            url_grouper = get_related_object(self.config, "url_grouper")
            if not url_grouper:
                return ""
            from djangocms_url_manager.models import Url

            url = Url.objects.filter(url_grouper=url_grouper).order_by("pk").last()
            if not url:  # pragma: no cover
                return ""
            return url.get_absolute_url() or ""

        from djangocms_link.helpers import get_link as djangocms_link_get_link

        return djangocms_link_get_link(self.config.get("link", {}), Site.objects.get_current().pk) or ""



from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase
from django.test import override_settings

from djangocms_frontend.contrib.tabs.cms_plugins import TabItemPlugin, TabPlugin
from djangocms_frontend.contrib.tabs.forms import TabForm, TabItemForm

from ..fixtures import TestFixture


class TabsPluginTestCase(TestFixture, CMSTestCase):
    def test_tab_plugin(self):
        add_plugin(
            placeholder=self.placeholder,
            plugin_type=TabPlugin.__name__,
            language=self.language,
        )
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "nav")

    def test_tab_item_plugin(self):
        parent = add_plugin(
            placeholder=self.placeholder,
            plugin_type=TabPlugin.__name__,
            language=self.language,
        ).initialize_from_form(TabForm)
        parent.save()
        add_plugin(
            target=parent,
            placeholder=self.placeholder,
            plugin_type=TabItemPlugin.__name__,
            language=self.language,
            config=dict(tab_title="tab title"),
        ).initialize_from_form(TabItemForm).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="tab-content">')
        self.assertContains(response, "tab-pane")

    @override_settings(
        DJANGOCMS_FRONTEND_TAB_TEMPLATES=(
            ("default", "Default"),
            ("custom", "Custom"),
        )
    )
    def test_tab_item_inherits_parent_template(self):
        """Test that TabItemPlugin correctly retrieves parent's template."""
        # Create a tab container with a custom template
        tab_container = add_plugin(
            placeholder=self.placeholder,
            plugin_type=TabPlugin.__name__,
            language=self.language,
            config=dict(template="custom"),
        )
        tab_container.initialize_from_form(TabForm).save()

        # Create a tab item as child of the tab container
        tab_item = add_plugin(
            target=tab_container,
            placeholder=self.placeholder,
            plugin_type=TabItemPlugin.__name__,
            language=self.language,
            config=dict(tab_title="Test Tab"),
        )
        tab_item.initialize_from_form(TabItemForm).save()

        # Get the plugin instance
        plugin_instance = TabItemPlugin()

        # Test that get_render_template uses the parent's template
        template_path = plugin_instance.get_render_template(
            context={}, instance=tab_item, placeholder=self.placeholder
        )

        # The template path should use 'custom' from the parent, not 'default'
        self.assertIn("custom", template_path)
        self.assertEqual(template_path, "djangocms_frontend/bootstrap5/tabs/custom/item.html")
